/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright (c) 2012 STRATO AG. All rights reserved.
 */
#include <sys/zfs_context.h>
#include <sys/errno.h>
#include <sys/fits_impl.h>

int
fits_add_count(fits_counter_t *fc, uint64_t ino, uint64_t inc,
	       uint64_t aux, uint64_t *new_count, uint64_t *old_aux)
{
	fits_count_elem_t *fce = fc->fc_head;

	while (fce && fce->fce_ino != ino)
		fce = fce->fce_next;

	if (!fce) {
		fce = kmem_alloc(sizeof(*fce), KM_SLEEP);
		fce->fce_ino = ino;
		fce->fce_count = 0;
		fce->fce_next = fc->fc_head;
		fc->fc_head = fce;
	}

	if (old_aux) {
		*old_aux = fce->fce_aux;
		fce->fce_aux = aux;
	}
	fce->fce_count += inc;

	if (new_count)
		*new_count = fce->fce_count;

	return 0;
}

int
fits_get_count(fits_counter_t *fc, uint64_t ino, uint64_t *new_count,
	       uint64_t *old_aux)
{
	fits_count_elem_t *fce = fc->fc_head;

	while (fce && fce->fce_ino != ino)
		fce = fce->fce_next;

	if (!fce) {
		if (new_count)
			*new_count = 0;
		if (old_aux)
			*old_aux = 0;
		return ENOENT;
	} else {
		if (new_count)
			*new_count = fce->fce_count;
		if (old_aux)
			*old_aux = fce->fce_aux;
	}

	return 0;
}

void
fits_free_count(fits_counter_t *fc, uint64_t ino)
{
	fits_count_elem_t *fce = fc->fc_head;
	fits_count_elem_t *prev = NULL;

	while (fce && fce->fce_ino != ino) {
		prev = fce;
		fce = fce->fce_next;
	}

	if (!fce)
		return;

	if (prev)
		prev->fce_next = fce->fce_next;
	else
		fc->fc_head = fce->fce_next;

	kmem_free(fce, sizeof(*fce));
}

int
fits_assert_count_empty(fits_counter_t *fc)
{
	fits_count_elem_t *fce = fc->fc_head;
	int ret = 0;

	while (fce) {
		printf("!fits_assert_count_empty: %s ino %"PRIu64" count %"PRIu64"\n",
		       fc->fc_name, fce->fce_ino, fce->fce_count);
		if (fce->fce_count != 0)
			++ret;
		fce = fce->fce_next;
		/* XXX TODO free count */
		/* XXX known leftover: if a file had > 1 links and be replaced
		 * by a file with > 1 link, but no same_name replacements, the
		 * link_add_cnt leaks with being 0
		 */
	}

	return ret;
}
